from typing import Generic, TypeVar

from .tlrparametres import TLRParametres

__all__ = ('TLParametres',)

ElementType = TypeVar('ElementType')


class TLParametres(
    Generic[ElementType]
):
    def __init__(
            self,
            tlr: TLRParametres[ElementType],
            size: int,
    ):
        assert isinstance(tlr, TLRParametres)
        assert isinstance(size, int)
        self.tlr = tlr
        assert size >= 0
        self.order = tlr.order
        assert isinstance(self.order, int)
        self.size = size
        self.height = 0
        while self._capacity() < size:
            self.height += 1
        self.capacity = self._capacity()
        assert isinstance(self.capacity, int)
        self.full = self.size == self.capacity
        self.leaf = self.height == 0
        self.subsize = self._subsize()
        assert isinstance(self.subsize, int)
        self.branching, self.lastsize, self.lastfull = self._branching_lastsize_lastfull()
        assert isinstance(self.branching, int)
        assert isinstance(self.lastsize, int)
        assert isinstance(self.lastfull, bool)

    def _branching_lastsize_lastfull(self) -> tuple[int, int, bool]:
        branching, lastsize = divmod(self.size, self.subsize)
        if lastsize == 0:
            return branching, self.subsize, True
        else:
            return branching + 1, lastsize, False

    def _capacity(self) -> int:
        return self.order ** (self.height + 1)

    def _subsize(self) -> int:
        return self.order ** self.height

    def superparams(self) -> 'TLParametres[ElementType]':
        return TLParametres(self.tlr, self.size + 1)

    def unit(self) -> 'TLParametres[ElementType]':
        return TLParametres(self.tlr, 1)

    def _subsize_no(self, index: int) -> int:
        assert isinstance(index, int)
        assert index < self.branching
        if index == self.branching - 1:
            return self.lastsize
        else:
            return self.subsize

    def params_no(self, index: int) -> 'TLParametres[ElementType]':
        assert isinstance(index, int)
        assert not self.leaf
        assert index < self.branching
        return TLParametres(
            self.tlr,
            self._subsize_no(index),
        )
